/* eslint-disable no-nested-ternary */
import React, { useState } from "react";
// import DateTimePicker from '@react-native-community/datetimepicker';
import DateTimePickerModal from "react-native-modal-datetime-picker";
import PropTypes from "prop-types";
import { Button, View, Text } from 'react-native'
import moment from "moment";
import { getDateFormat, resolveElixirDate,evalStyle } from "@muhammadz/elixir-react-stateful/helpers/util";
import { getTranslation } from '../../labels'
import { Styles } from "../../src-screens/Styles/eliraStyles";

const ButtonLabel = {
  'Date': 'dateButton',
  'Time': 'timeButton',
  'DateTime': 'datetimeButton'
}

const DateComponent = ({ meta, value, onChange, timeZoneOffset, isSearch, onKeyUp, theme, customStyle, path, tdlPath }) => {
  const [isDatePickerVisible, setDatePickerVisibility] = useState(false);

  const showDatePicker = () => {
    setDatePickerVisibility(true);
  };
  const hideDatePicker = () => {
    setDatePickerVisibility(false);
  };
  // evalStyle(customStyle, value, meta)
  const { type, isMandatory } = meta;
  const format = getDateFormat(type);
  // const dateValue = moment(value)
  const dateValue = resolveElixirDate(value, type, timeZoneOffset);
  let dateStyle = theme.keyDate;

  if (customStyle) {
    dateStyle = { ...dateStyle, ...customStyle }
  }

  return (
    <View style={{ flex:theme.keyView.flex, flexDirection: theme.keyView.flexDirection }}>
      <View style={[dateStyle, { justifyContent: theme.keyView.justifyContent }]}>
        <Text stlye={dateStyle}>{dateValue && dateValue.format(format)}</Text>
        <Button stlye={{ flex: theme.keyButton.flex }} title={getTranslation(ButtonLabel[type])} onPress={showDatePicker} />
      </View>

      <DateTimePickerModal
        style={[Styles.borderStyle,dateStyle, {
          marginLeft: theme.keyDateTimePicker.marginLeft,
          marginRight: keyDateTimePicker.marginRight,

        }]}
        isVisible={isDatePickerVisible}
        mode={type === "Time" ? "time" : type === "DateTime" ? "datetime" : "date"}
        onConfirm={(val) => {
          hideDatePicker();
          const dateObject = new Date(val)
          // change timezone from client to server by removing the timezone of the client and adding that of the server
          // the dateObject.getTimezoneOffset() has - before it because it returns the -timezone so to subtract it we must add. this is not a bug
          // this is only done when we have time value. meaning only in (DateTime and Time)
          if (type !== "Date") {
            dateObject.setMinutes((dateObject.getMinutes() + dateObject.getTimezoneOffset()) + (timeZoneOffset / 60000));
          }
          val = moment(dateObject)
          onChange(val ? val.format(format) : null);

        }}
        onCancel={hideDatePicker}
      />
    </View>
    // <DateTimePickerModal
    //   value={10000.0}
    // mode={type==="Time"?"time":"date"}
    // is24Hour={true}
    // display="default"
    //   onChange={(val) => {
    //     onChange(val ? val.format(format) : null);
    //   }}
    // />
  );


};

DateComponent.propTypes = {
  meta: PropTypes.objectOf(PropTypes.any).isRequired,
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  onChange: PropTypes.func.isRequired,
  timeZoneOffset: PropTypes.number.isRequired,
  isSearch: PropTypes.bool.isRequired,
  onKeyUp: PropTypes.func,
  theme: PropTypes.objectOf(PropTypes.any),
  customStyle: PropTypes.objectOf(PropTypes.any)
};

DateComponent.defaultProps = {
  value: null,
  onKeyUp: null,
};

export default DateComponent;
