import React, { useRef } from "react";
import MultiSelect from 'react-native-multiple-select';
import PropTypes from "prop-types";
import { View, Text } from 'react-native';
import * as colors from './../../src-screens/environments/colors'
import { Styles } from "../../src-screens/Styles/eliraStyles";
import { evalStyle } from "../../helpers/util";
const DropDownComponent = ({ meta, value, onChange, onClear, theme, customStyle, path, tdlPath }) => {
  const { pv, isMandatory } = meta;
  const multiSelect = useRef(null)
  // evalStyle(customStyle, value, meta)
  let inputGroupStyle = {
    backgroundColor: theme.keyInputItem.backgroundColor,
    paddingLeft: theme.keyInputItem.paddingLeft,
    elevation: theme.keyInputItem.elevation,
    borderRadius: theme.keyInputItem.borderRadius,
    marginLeft: theme.keyInputItem.marginLeft,
    marginRight: theme.keyInputItem.marginRight
  }
  if (customStyle) {
    inputGroupStyle = { ...inputGroupStyle, ...customStyle.inputGroupStyle }
  }

  let listContainerStyle = {

    elevation: theme.keyListContainer.elevation,
    backgroundColor: colors.Geyser,
    padding:theme.keyListContainer.padding,
    borderWidth:theme.keyListContainer.borderWidth,
    borderRadius:theme.keyListContainer.borderRadius,
    marginLeft:theme.keyListContainer.marginLeft,
    marginRight: theme.keyListContainer.marginRight

  }

  if (customStyle) {
    listContainerStyle = { ...listContainerStyle, ...customStyle.listContainerStyle }
  }

  let rowListStyle = {
    alignContent:theme.keyRowList.alignContent,
    justifyContent: theme.keyRowList.justifyContent,
    elevation: theme.keyRowList.elevation,
     borderColor: colors.WHITE,
    borderWidth: theme.keyRowList.borderWidth,
     borderRadius: theme.keyRowList.borderRadius,
  }

  if (customStyle) {
    rowListStyle = { ...rowListStyle, ...customStyle.rowListStyle }
  }

  let dropDownMenuStyle = theme.keyDropdownMenu;

  if (customStyle) {
    dropDownMenuStyle = { ...dropDownMenuStyle, ...customStyle.dropDownMenuStyle }

  }

  return (

    <>
      <MultiSelect
        hideTags
        items={pv}
        uniqueKey="pk"
        ref={(component) => { multiSelect.current = component }}
        onSelectedItemsChange={(val) => {
          val = val[0]
          onChange(val);
        }}
        onClearSelector={onClear}
        selectText={value && value._label || value}
        selectedItems={[value && (value.pk || value)]}
        altFontFamily="ProximaNova-Light"
        tagRemoveIconColor="#CCC"
        tagBorderColor="#CCC"
        tagTextColor="#CCC"
        displayKey="_label"
        searchInputStyle={{ color: '#CCC', }}
        submitButtonColor="#CCC"
        styleDropdownMenu={[Styles.borderStyle, dropDownMenuStyle,]}
        itemFontSize={14}
        fontSize={Styles.fontSize}
        selectedItemTextColor={colors.Cutty_Sark}
        selectedItemIconColor={colors.Cutty_Sark}
        itemTextColor={colors.BLACK}
        textColor={colors.DARK_GRAY}
        styleListContainer={[listContainerStyle,]}
        styleRowList={rowListStyle}
        styleItemsContainer={{ padding: 5, }}
        styleInputGroup={inputGroupStyle}
        single={true}
      />
      <View>
        {multiSelect.current != null && multiSelect.current.getSelectedItemsExt(pv)}
      </View>
    </>

    // <Select
    //   showSearch
    //   allowClear={isMandatory === "false"}
    //   onClear={onClear}
    //   style={{ width: "100%" }}
    //   value={value && (value.pk || value)}
    //   onChange={onChange}
    //   onSelect={onChange}
    //   filterOption={(input, option) => {
    //     return option.children.includes(input);
    //   }}
    //   onKeyUp={onKeyUp}
    // >
    //   {pv &&
    //     pv.map(({ pk, _label }) => (
    //       <Select.Option key={pk} value={pk} title={_label}>
    //         {_label}
    //       </Select.Option>
    //     ))}
    // </Select>
  );
};

DropDownComponent.propTypes = {
  meta: PropTypes.objectOf(PropTypes.any).isRequired,
  value: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  onChange: PropTypes.func.isRequired,
  onClear: PropTypes.func.isRequired,
  theme: PropTypes.objectOf(PropTypes.any),
  customStyle: PropTypes.objectOf(PropTypes.any)
};

DropDownComponent.defaultProps = {
  value: null
};

export default DropDownComponent;
