import React from "react";
import PropTypes from "prop-types";
import { Tooltip } from "antd";
import * as Util from "../helpers/util";
import HTMLValue from "./htmlValue";

const cellSize = 50;

const TableRow = ({ object, meta, rowsColor }) => {
  const { keyName } = meta;
  const objectType = Util.simpleTypes.indexOf(meta.type) !== -1 ? Util.Simple : Util.Composition;
  const path = objectType === Util.Simple ? keyName + Util.serverFormatPostfix : keyName;
  const currentValue = Util.getObjectWithPath(object, path);
  if (currentValue === undefined || currentValue === null) return "";
  if (typeof currentValue === "object" && Util.isEmpty(currentValue)) {
    return "";
  }
  if (currentValue._label) return currentValue._label;
  if (currentValue && typeof currentValue === "string") {
    const trimmedValue = `${currentValue.substr(0, cellSize)} ${
      currentValue.length > cellSize ? "...." : ""
    }`;
    if (currentValue.length > cellSize) {
      return (
        <Tooltip title={currentValue}>
          <span style={{ color: rowsColor[object.pk] }}>
            {Util.isHTML(currentValue) ? trimmedValue : <HTMLValue value={trimmedValue} />}
          </span>
        </Tooltip>
      );
    }
  }
  return <span style={{ color: rowsColor[object.pk] }}>{currentValue}</span>;
};

TableCell.propTypes = {
  // eslint-disable-next-line react/forbid-prop-types
  object: PropTypes.any.isRequired,
  meta: PropTypes.objectOf(PropTypes.any).isRequired,
  rowsColor: PropTypes.objectOf(PropTypes.any),
};

TableCell.defaultProps = {
  rowsColor: {},
};

export default TableCell;
