import React, { useEffect, useState, useRef } from "react";
import PropTypes from "prop-types";
import FlashMessage from "react-native-flash-message";
import { connect } from "react-redux";
import * as Updates from "expo-updates";
import * as Util from "../../helpers/util";
import { ActivityIndicator, Drawer, List, Button } from "@ant-design/react-native";
import "react-native-get-random-values";
import { v4 as uuidv4 } from "uuid";
import { View, ScrollView, Text, StyleSheet, I18nManager, Icon, StatusBar, Platform } from "react-native";
import * as LocalStorage from "../../service/localStorage";
import SmsTDL from "./sms";
import CevTDL from "./cev";
import GotoTDL from "./goto";
import TdlHeader from "./tdlHeader/index";
import * as colors from "../../src-screens/environments/colors";
import {
  sendRequest,
  executeAction,
  setIsDrawerOpen,
  setCurrentPassword,
  setCurrentUser,
  login,
} from "../../redux/actions";
import { showNotification } from "../../config/notifications";
import { getTranslation } from "../../labels";
import store from "../../redux";
import { getTheme } from "../../styles/themes";
import * as eliraUtil from "../../src-screens/eliraUtil/eliraUtil";
import { theme } from "native-base";
import { SwipeableDrawer } from "@mui/material";
import { KeyboardAwareScrollView } from "react-native-keyboard-aware-scrollview";
import TdlFooter from "./tdlFooter";
import { LeftOutlined, MoreOutlined, RightOutlined } from "@ant-design/icons";
import RBSheet from "react-native-raw-bottom-sheet";
import {TouchableOpacity} from "react-native"
const tdlTheme = getTheme("default");
const styles = StyleSheet.create({
  container: {
    flex: tdlTheme.tdlContainer.flex,
  },
  webDrawer: {
    width: 250
  }
});

const drawerStyles = {
  drawer: {
    shadowColor: tdlTheme.tdlDrawer.shadowColor,
    shadowOpacity: tdlTheme.tdlDrawer.shadowOpacity,
    shadowRadius: tdlTheme.tdlDrawer.shadowRadius,

  },
  main: { paddingLeft: tdlTheme.tdlMain.paddingLeft, },
};

const compareTDLState = (oldState, newState) => {
  console.log(oldState.tabs.length);
  console.log(newState.tabs.length);
  let isEqual = Util.isEqual(oldState, newState);
  if (!Util.isEqual(oldState.tabs, newState.tabs)) isEqual = false;
  console.log("isEqual = " + isEqual);
  return isEqual;
}




// SIDEBAR
const createDrawerContent = ({refRBSheet, viewStatus, dispatch, onSuccessCallbacks, clientActions } = () => { }) => {
  const actions = viewStatus.getHeaderActions();
  const allActions = actions[0].concat(actions[1]);
  const countOfActions = allActions.length;
  // console.log({ allActions })
  const headerButtons = [];
  headerButtons.push(
    allActions.map((action) => (
      // <List.Item
      //   key={action.actionName}
      //   multipleLine
      //   onPress={() => {
      //     const callback = (obj) => {
      //       if (onSuccessCallbacks && onSuccessCallbacks[action.tdlId]) {
      //         onSuccessCallbacks[action.tdlId]({ ...obj, store });
      //       }
      //     };
      //     dispatch(executeAction(viewStatus.usecaseId, action.tdlId, action.responseType, callback));
      //     dispatch(setIsDrawerOpen(false));
      //   }}


      // >
      //   <View
      //     style={tdlTheme.tdlViewActions}>
      //     <Text>{action.actionLabel}</Text>
      //   </View>
      // </List.Item>
      <>
        {countOfActions <= 2 &&
          <View style={{ flex: 1 }}>
            <Button
              onPress={() => {
                const callback = (obj) => {
                  if (onSuccessCallbacks && onSuccessCallbacks[action.tdlId]) {
                    onSuccessCallbacks[action.tdlId]({ ...obj, store });
                  }
                };
                dispatch(executeAction(viewStatus.usecaseId, action.tdlId, action.responseType, callback));
                dispatch(setIsDrawerOpen(false));
              }}
              style={{ borderColor: "#468ca8" }}
            >
              {action.actionLabel}
            </Button>
          </View>
        }

      </>


    )),
  );

  if (countOfActions > 2) {
    headerButtons.push(
      <>
        <View style={{ flex: 1 }}>
          <Button
            onPress={() => {
              const callback = (obj) => {
                if (onSuccessCallbacks && onSuccessCallbacks[allActions[0].tdlId]) {
                  onSuccessCallbacks[allActions[0].tdlId]({ ...obj, store });
                }
              };
              dispatch(executeAction(viewStatus.usecaseId, allActions[0].tdlId, allActions[0].responseType, callback));
              dispatch(setIsDrawerOpen(false));
            }}
            style={{ borderColor: "#468ca8" }}
          >
            {allActions[0].actionLabel}
          </Button>
        </View>
        <View style={{ flex: 2 }}>
          <Button
            onPress={() => {
              const callback = (obj) => {
                if (onSuccessCallbacks && onSuccessCallbacks[allActions[1].tdlId]) {
                  onSuccessCallbacks[allActions[1].tdlId]({ ...obj, store });
                }
              };
              dispatch(executeAction(viewStatus.usecaseId, allActions[1].tdlId, allActions[1].responseType, callback));
              dispatch(setIsDrawerOpen(false));
            }}
            style={{ borderColor: "#468ca8" }}
          >
            {allActions[1].actionLabel}
          </Button>
        </View>
        <View style={{ flex: 1 }}>
          <Button
            style={{ borderColor: "#468ca8" }}
            onPress={() => {
              refRBSheet.current.open();
            }}
          >
            {Platform.OS == "web" &&
              <MoreOutlined />
            }
          </Button>
        </View>
      </>
    )
  }

  if (viewStatus && clientActions[viewStatus.tdl.tdlId]) {
    headerButtons.push(
      Object.entries(clientActions[viewStatus.tdl.tdlId]).map((action) => {
        action = action[1];
        return (
          action.position === "drawer" && (
            <List.Item
              key={action.actionName}
              multipleLine
              onPress={() => {
                action.execution({ store, vs: viewStatus });
              }}
            >
              <View style={tdlTheme.tdlViewActions} >
                <Text>{action.actionName}</Text>
              </View>
            </List.Item>
          )
        );
      }),
      // ),
    );
  }
  const sidebar = (
    <>
      {headerButtons}
    </>


  );
  return sidebar;
};

const drawMoreActions = ({ viewStatus, dispatch, onSuccessCallbacks, clientActions } = () => { }) => {
  const actions = viewStatus.getHeaderActions();
  const allActions = actions[0].concat(actions[1]);
  console.log("try allActions",allActions)
  const headerButtons = [];
  headerButtons.push(
    allActions.map((action) => {

      return (
        <TouchableOpacity
          onPress={() => {
            const callback = (obj) => {
              if (onSuccessCallbacks && onSuccessCallbacks[action.tdlId]) {
                onSuccessCallbacks[action.tdlId]({ ...obj, store });
              }
            };
            dispatch(executeAction(viewStatus.usecaseId, action.tdlId, action.responseType, callback));
            dispatch(setIsDrawerOpen(false));
          }}

        >
          <View
            style={{
              // alignSelf: "auto",
              borderBottomWidth: 0.5,
              borderStyle: "solid",
              borderColor: colors.GRAY,
              height:60,
              // backgroundColor:"pink",
              alignItems: "center",
              padding:10
            }}
          >

            <Text  style={{ alignSelf: "auto", textAlign: "center"}}>
              {action.actionLabel}
            </Text>

          </View>
        </TouchableOpacity>
      );

    })
)

const sidebar = (
  <>
    {headerButtons}
  </>


);
return sidebar;

}
const footerDrawer = ({ refRBSheet,viewStatus, dispatch, onSuccessCallbacks, clientActions, customStyle, setPageNumber, pageNumber, sliceNumber } = () => { }) => {
  return (
    <>
      <View style={{ display: "flex", flexDirection: "row", marginBottom: 20 }}>
        <View style={{ flex: 1 }}>
          <Button style={{ backgroundColor: "#8da3ac" }} onPress={() => {
            console.log(`sliceNumber of ${sliceNumber} type of ${typeof sliceNumber}`)
            console.log(`sliceNumber1 of ${pageNumber} type of ${typeof pageNumber}`)
            if (sliceNumber > pageNumber)
              setPageNumber(++pageNumber)
          }}>
            <RightOutlined />
          </Button>
        </View>
        <View style={{ flex: 2, display: "flex", flexDirection: "row" }}>
          {/* <View  style={{ flex: 1}}>
                <Button >14</Button>
            </View>
            <View style={{ flex: 1}}>
                <Button >15</Button>
            </View> */}
          {createDrawerContent({ refRBSheet,viewStatus, dispatch, onSuccessCallbacks, clientActions, customStyle })}
        </View>
        <View style={{ flex: 1 }}>
          <Button style={{ backgroundColor: "#8da3ac" }} onPress={() => {
            if (pageNumber > 0)
              setPageNumber(--pageNumber)
          }}>
            <LeftOutlined />
          </Button>
        </View>
      </View>
    </>
  )
}

const tdlContent = (viewStatus, dispatch, timeZoneOffset, isRTL, themeName, onSuccessCallbacks, clientActions, customStyle, TDLComponent, pageNumber) => {
  console.log("pagNumber", pageNumber)
  return <>
    {(viewStatus.tdl.type == "CEV" && (
      <KeyboardAwareScrollView >
        <TDLComponent
          vs={viewStatus}
          dispatch={dispatch}
          timeZoneOffset={timeZoneOffset}
          isRTL={isRTL}
          themeName={themeName}
          onSuccessCallbacks={onSuccessCallbacks}
          clientActions={clientActions}
          customStyle={customStyle}
          isLoading={viewStatus.isLoading}
          pageNumber={pageNumber}

        />
      </KeyboardAwareScrollView>
    )) || (
        <TDLComponent
          vs={viewStatus}
          dispatch={dispatch}
          timeZoneOffset={timeZoneOffset}
          isRTL={isRTL}
          themeName={themeName}
          onSuccessCallbacks={onSuccessCallbacks}
          clientActions={clientActions}
          customStyle={customStyle}
          pageNumber={pageNumber}
        />
      )}

  </>

}

const getBottomClientActions = ({ viewStatus, clientActions, dispatch, theme }) => {
  return (
    viewStatus &&
    clientActions[viewStatus.tdl.tdlId] &&
    Object.entries(clientActions[viewStatus.tdl.tdlId]).map((action) => {
      if (action.position === "bottom")
        return (
          <Button
            style={theme.footerButton}
            transparent
            onPress={() => {
              action.execution({ dispatch, vs: viewStatus });
            }}
          >
            <Text style={theme.footerButton}>{action.actionName}</Text>
          </Button>
        );

      return <></>;
    })
  );
};

//TDL Component

const TDL = ({
  vs,
  tabs,
  dispatch,
  favorite,
  timeZoneOffset,
  tdlId,
  viewStatuses,
  smallView,
  isDrawerOpen,
  isRTL,
  onSuccessCallbacks,
  userName,
  password,
  themeName,
  onTDLInit,
  clientActions,
  customStyle,
  isSessionInvalidated,
  isLoading,
  showHeader
}) => {
  const [tabId] = useState(uuidv4());
  const [viewStatus, setViewStatus] = useState(vs);
  const [draw, setDraw] = useState(false);
  const [pageNumber, setPageNumber] = useState(0);
  const refRBSheet = useRef();
  let sliceNumber = 0;
  if (viewStatus)
    if (viewStatus.tdl)
      if (viewStatus.tdl.groups[0].subGroups) {
        console.log("nmjccjj", viewStatus.tdl.groups[0].subGroups)
        sliceNumber = viewStatus.tdl.groups[0].subGroups.length - 1;
      }
  const successHandler = (response) => {
    if (response) {
      const { status } = response;
      if (status === 200) {
        showNotification({
          message: getTranslation("loginMessage"),
          description: getTranslation("loginDescription").replace("{username}", response.data.userName),
          type: "success",
        });
      }
    }
  };



  const loginHandler = () => {
    dispatch(setCurrentPassword({ currentPassword: password }));
    dispatch(setCurrentUser({ username: userName }));
    if (!eliraUtil.checkUserIsGuest(userName) && isSessionInvalidated)
      dispatch(login(userName, password, successHandler));
  };

  const drawer = useRef(null);
  useEffect(() => {
    loginHandler(userName, password);
  }, []);

  useEffect(() => {
    if (tdlId) {
      const callback = (obj) => {
        if (onSuccessCallbacks && onSuccessCallbacks[tdlId]) {
          onSuccessCallbacks[tdlId]({ ...obj, store, setDraw });
        } else {
          setDraw(true);
        }
      };
      dispatch(
        sendRequest(
          { tdlId },
          { withLoader: false, newTab: true, drawUsecaseId: true, mainTab: false, tabId },
          { username: userName, password },
          // onSuccessCallbacks[tdlId],
          callback,
        ),
      );
    }
  }, []);

  useEffect(() => {
    setViewStatus(vs);
  }, [vs]);

  useEffect(() => {
    const id = (vs && vs.tdlId) || tdlId;
    if (onTDLInit[id]) {
      onTDLInit[id]({ dispatch, vs });
    }
  }, []);

  useEffect(() => {
    if (!vs) {
      const currentTab = tabs.find((item) => item.id === tabId);
      if (currentTab) {
        setViewStatus(store.getState().viewStatus[currentTab.usecaseId]);
      }
    }
  }, [tabs]);

  const [storageRTL, setStorageRTL] = useState(null);

  useEffect(() => {
    (async () => {
      await LocalStorage.getisRTL().then((value) => {
        value = JSON.parse(value);
        setStorageRTL(value);
        LocalStorage.setisRTL(isRTL);
        if (value != isRTL) {
          I18nManager.forceRTL(isRTL);
          // Updates.reloadAsync();
        }
      });
    })();
  }, []);

  if (!viewStatus) return null;

  const getTdlFactory = () => {
    if (!viewStatus) {
      return null;
    }
    const { type } = viewStatus.tdl;
    if (type === "SMS") return SmsTDL;
    if (type === "CEV") return CevTDL;
    return GotoTDL;
  };





  const sidebar = draw && createDrawerContent({ viewStatus, dispatch, onSuccessCallbacks, clientActions, customStyle });
  // get TDL COmponent
  const TDLComponent = draw && getTdlFactory();
  console.log("isDrawerOpen", viewStatus.isLoading)
  const changeCounter = (number) => {
    // setCount(number)
    console.log("nummmmber", number)
  }
  return (
    <>
     <RBSheet
        height={400}
        ref={refRBSheet}
        closeOnDragDown={true}
        closeOnPressMask={true}
        customStyles={{
          wrapper: {
            backgroundColor: colors.TRANSPARENT,
          },
          draggableIcon: {
            backgroundColor: colors.BLACK,
          },
          container: {
            padding: 5,
          },
        }}
      >
        <ScrollView style={{ height: 60,/* alignSelf: "auto"*/ }}>
         {drawMoreActions({ viewStatus, dispatch, onSuccessCallbacks, clientActions, customStyle, setPageNumber, pageNumber, sliceNumber })}
        </ScrollView>
      </RBSheet>

      <ActivityIndicator animating={viewStatus.isLoading} toast size="large" text="Loading..." />

      {!viewStatus.isPopUp() && draw && (
        <TdlHeader
          dispatch={dispatch}
          favorite={favorite}
          vs={viewStatus}
          timeZoneOffset={timeZoneOffset}
          smallView={smallView}
          isRTL={isRTL}
          clientActions={clientActions}
          themeName={themeName}
          customStyle={{}}
          showHeader={showHeader}
        />
      )}

      {draw && (
        <>
          {Platform.OS == "web" &&
            <>
              <SwipeableDrawer
                anchor={isRTL ? "right" : "left"}
                // open={state[anchor]}
                onClose={(open) => dispatch(setIsDrawerOpen(false))}
                onOpen={(open) => dispatch(setIsDrawerOpen(open))}
                open={isDrawerOpen}
              // onOpenChange={(open) => dispatch(setIsDrawerOpen(open))}
              // drawerRef={(el) => (drawer.current = el)}
              // style={{ backgroundColor: "#ccc", width: 200 }}
              >
                {sidebar}
              </SwipeableDrawer>
              {tdlContent(viewStatus, dispatch, timeZoneOffset, isRTL, themeName, onSuccessCallbacks, clientActions, customStyle, TDLComponent, pageNumber, setPageNumber)}
            </>

          }
          {Platform.OS != "web" &&
            <Drawer
              sidebar={sidebar}
              position={isRTL === true ? "left" : "right"}
              open={isDrawerOpen}
              onOpenChange={(open) => dispatch(setIsDrawerOpen(open))}
              drawerRef={(el) => (drawer.current = el)}
              drawerBackgroundColor="#ccc"
            // style={}
            >

              {tdlContent(viewStatus, dispatch, timeZoneOffset, isRTL, themeName, onSuccessCallbacks, clientActions, customStyle, TDLComponent, setPageNumber, pageNumber)}

            </Drawer>

          }

          <View style={tdlTheme.tdlViewActionBottom}>
            {getBottomClientActions({ viewStatus, dispatch, clientActions, theme: getTheme(themeName) })}
          </View>
        </>
      )}
      {/* <TdlFooter
        dispatch={dispatch}
        favorite={favorite}
        vs={viewStatus}
        timeZoneOffset={timeZoneOffset}
        smallView={smallView}
        isRTL={isRTL}
        clientActions={clientActions}
        themeName={themeName}
        customStyle={{}}
        showHeader={showHeader}
      /> */}
      {footerDrawer({ refRBSheet,viewStatus, dispatch, onSuccessCallbacks, clientActions, customStyle, setPageNumber, pageNumber, sliceNumber })}
     
    </>
  );
};

TDL.propTypes = {
  vs: PropTypes.objectOf(PropTypes.any),
  tdlId: PropTypes.string,
  dispatch: PropTypes.func.isRequired,
  favorite: PropTypes.arrayOf(PropTypes.any).isRequired,
  timeZoneOffset: PropTypes.number.isRequired,
  viewStatuses: PropTypes.objectOf(PropTypes.any).isRequired,
  smallView: PropTypes.bool,
  tabs: PropTypes.arrayOf(PropTypes.objectOf(PropTypes.any)).isRequired,
  isDrawerOpen: PropTypes.bool.isRequired,
  isRTL: PropTypes.bool.isRequired,
  onSuccessCallbacks: PropTypes.objectOf(PropTypes.any),
  userName: PropTypes.string.isRequired,
  password: PropTypes.string.isRequired,
  themeName: PropTypes.string,
  onTDLInit: PropTypes.objectOf(PropTypes.any),
  clientActions: PropTypes.objectOf(PropTypes.any),
  customStyle: PropTypes.objectOf(PropTypes.any),
};

TDL.defaultProps = {
  tdlId: null,
  vs: null,
  smallView: false,
  onSuccessCallbacks: {},
  themeName: "default",
  onTDLInit: {},
  clientActions: {},
  customStyle: {},
};

const mapStateToProps = (state) => ({
  viewStatuses: state.viewStatus,
  // vs: state.vs,
  tabs: state.tabs.tabs,
  favorite: state.general.favorite,
  timeZoneOffset: state.general.timeZoneOffset,
  isDrawerOpen: state.general.isDrawerOpen,
  isRTL: state.general.isRTL,
  isSessionInvalidated: state.general.isSessionInvalidated,
  // isLoading: state.general.isLoading,
});

export default connect(mapStateToProps)(React.memo(TDL));
